# Copyright [2019] [Christopher Syben, Markus Michen]
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import numpy as np
from pyronn.ct_reconstruction.geometry.geometry_cone_3d import GeometryCone3D
from pyronn.ct_reconstruction.helpers.trajectories import circular_trajectory


 ##   This file defines the Geometry parameters used by the whole model.
 ##   A GeometryParallel2D instance is provided to be used by everyone that needs it.

# Declare Parameters
volume_shape          = [100, 512, 512]
volume_spacing        = [0.5, 0.5, 0.5]

detector_shape        = [512, 384]
detector_spacing      = [0.8, 0.8]

number_of_projections = 360
angular_range = 2 * np.pi

source_detector_distance = 900
source_isocenter_distance = 560

# Create Geometry class instance
GEOMETRY = GeometryCone3D(volume_shape, volume_spacing, detector_shape,
                        detector_spacing, number_of_projections, angular_range,
                      source_detector_distance, source_isocenter_distance)
GEOMETRY.set_trajectory(circular_trajectory.circular_trajectory_3d(GEOMETRY))


