# Copyright [2019] [Christopher Syben, Markus Michen]
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import tensorflow as tf
import numpy as np
from .geometry_parameters import GEOMETRY
from pyronn.ct_reconstruction.helpers.filters.filters import ramp
from pyronn.ct_reconstruction.layers.backprojection_3d import cone_backprojection3d
from tensorflow.keras import Model

class filter_model(Model):
    """
            Here the model in terms of Tensorflow layers is defined.
        """

    def __init__(self):
        super(filter_model, self).__init__()
        filter = ramp(GEOMETRY.detector_shape[np.alen(GEOMETRY.detector_shape)-1])
        # filter = ramp_3D(GEOMETRY)
        self.filter_weights = tf.Variable(name='filter_frequency', dtype=tf.float32, initial_value=filter, trainable=True)  # init as ramp filter


    def get_filter(self, sess):
        return sess.run(self.filter_weights)

    def set_filter(self, sess, filter_weights):
        sess.run(self.set_filter_weights, feed_dict={self.filter_weights_placeholder: filter_weights})

    def call(self, x):
        """
                Sets up the network architecture.

                Args:
                images_batch: batch input of training data

                Returns:
                backprojection_layer: The last layer before the loss layer.
                """


        sinogram_frequency = tf.signal.fft(tf.cast(x,dtype=tf.complex64))

        #print("sinogram_frequency:", sinogram_frequency.shape, "self.filter_weights:", self.filter_weights.shape)

        filtered_sinogram_frequency = tf.multiply(sinogram_frequency, tf.cast(self.filter_weights, dtype=tf.complex64))

        #print("filtered_sinogram_frequency:", filtered_sinogram_frequency.shape)

        filtered_sinogram = tf.math.real(tf.signal.ifft(filtered_sinogram_frequency))

        reco = cone_backprojection3d(filtered_sinogram, GEOMETRY)

        tf.nn.relu(reco)
        return reco
