# Copyright [2019] [Christopher Syben, Markus Michen]
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


# 屏蔽掉TensorFlow的加载信息
import os
os.environ["TF_CPP_MIN_LOG_LEVEL"] = "2"

import tensorflow as tf
import numpy as np
import matplotlib.pyplot as plt

from model import training_parameter as args

from pyronn.ct_reconstruction.layers.projection_3d import cone_projection3d
from pyronn.ct_reconstruction.layers.backprojection_3d import cone_backprojection3d
from pyronn.ct_reconstruction.geometry.geometry_cone_3d import GeometryCone3D
from model.geometry_parameters import GEOMETRY
from model.pipeline import pipeline
from pyronn.ct_reconstruction.helpers.filters.filters import ram_lak_3D
from model.input_data import get_test_data
from plots import evaluation

# os_path_data = r"/home/bme/Documents/samples_labels/S2CTVolu512x512x100.raw"
os_path_proj_train = r"/home/bme/Documents/samples_labels/proj_log.dat"
os_path_reco_train = r"/home/bme/Documents/samples_labels/Output_2e3.dat"

os_path_proj_val = r"/home/bme/Downloads/qq-files/2723609243/file_recv/projection_sc_log.dat"
os_path_reco_val = r"/home/bme/Downloads/qq-files/2723609243/file_recv/Output_sc_2e3.dat"

proj_train = np.fromfile(os_path_proj_train, dtype="float32")
proj_train = tf.reshape(proj_train, [-1, 512, 384])
proj_train = np.array(proj_train)

proj_val = np.fromfile(os_path_proj_val, dtype="float32")
proj_val = tf.reshape(proj_val, [-1, 512, 384])


reco_train = np.fromfile(os_path_reco_train, dtype="float32")
reco_train = tf.reshape(reco_train, [-1, 512, 512])
reco_train = np.array(reco_train)

reco_val = np.fromfile(os_path_reco_val, dtype="float32")
reco_val = tf.reshape(reco_val, [-1, 512, 512])

# print(data_val.shape, label_val.shape, data_test.shape, label_test.shape)
# data_cupping, label_cupping = get_test_cupping_data()

data = proj_train
label = reco_train

data_val = proj_val
label_val = reco_val

# data_test, label_test = get_test_data(args.NUM_TEST_SAMPLES)

# expand dimension ：
data = np.broadcast_to(data, [args.BATCH_SIZE_TRAIN, *np.shape(data)])
label = np.broadcast_to(label, [args.BATCH_SIZE_TRAIN, *np.shape(label)])

data_val = np.broadcast_to(data_val, [args.BATCH_SIZE_VALIDATION, *np.shape(data_val)])
label_val = np.broadcast_to(label_val, [args.BATCH_SIZE_VALIDATION, *np.shape(label_val)])

#data_test = np.broadcast_to(data_test, [args.NUM_TEST_SAMPLES, *np.shape(data_test)])
#label_test = np.broadcast_to(label_test, [args.NUM_TEST_SAMPLES, *np.shape(label_test)])

print(data.shape, label.shape, data_val.shape, label_val.shape)

training = pipeline()
training.train(data, label, data_val, label_val)

initial_filter = training.results.get('initial_filter')
learned_filter = training.results.get('learned_filter')
ram_lak_filter = ram_lak_3D(GEOMETRY) # shape:(360, 1, 384)

result_test_initial = training.forward(data_test, label_test, initial_filter)
result_test_learned = training.forward(data_test, label_test, learned_filter)
result_test_ram_lak = training.forward(data_test, label_test, ram_lak_filter)


# result_cupping_test_initial = training.forward(data_cupping, label_cupping, initial_filter)
# result_cupping_test_ram_lak = training.forward(data_cupping, label_cupping, ram_lak_filter)


evaluation.evaluation_filter(initial_filter, ram_lak_filter, learned_filter, 'plots/filter.pdf')
evaluation.evaluation_three(result_test_initial, result_test_ram_lak, result_test_learned, GEOMETRY.volume_shape, 'plots/test.pdf')
# valuation.evaluation_three(result_cupping_test_initial, result_cupping_test_ram_lak, result_cupping_test_learned, GEOMETRY.volume_shape, 'plots/cupping.pdf')

